#pragma once
#ifndef _CPISP_SETTING_H
#define  _CPISP_SETTING_H
#include "CpDbBase.h"
#include <map>
#include "../../Tool_Common/CpArgDefine.h"

typedef struct STRUCT_RESERVE_ITEM
{
    BYTE ucBankIndex;
    BOOL bSave;
    // Save
    WORD usStartAddr;
    DWORD ulLen;
    // Sector erase
    BYTE ucEraseSectorCount;
    BOOL bAuto;
}STRUCT_RESERVE_ITEM;

typedef enum
{
    _ENUM_SPECIFIED_ONE = 0,
    _ENUM_DETECTED_ONE,
    _ENUM_CANCEL,
}ENUM_SPECIFY_FLASH_MISMATCH_DETECTED;

typedef enum
{
    _ENUM_FLASH_CHIP_ERASE = 0,
    _ENUM_FLASH_BANK_ERASE,
    _ENUM_FLASH_SECTOR_ERASE,
}ENUM_ERASE_FLASH_TYPE;

//////////////////////////////////////////////////////////////////////////
typedef enum
{
    _ENUM_EEPROM_DIRECT = 0,
    _ENUM_EEPROM_THROUGH_IC,
    _ENUM_FLASH,
}STRUCT_ENUM_DEVICE_TYPE;

typedef enum
{
    _ENUM_EEPROM_ADDR_BYTE = 1,
    _ENUM_EEPROM_ADDR_WORD,
    _ENUM_EEPROM_ADDR_SEGMENT,
}STRUCT_ENUM_EEPROM_ADDR_LEN;

typedef enum
{
    _ENUM_THROUGH_FW = 0, //by Sys eeprom
    _ENUM_GPIO_I2C,
    _ENUM_THROUGH_DDCCI,
    _ENUM_BYPASS,
    _ENUM_THROUGH_FW_OTHER_IIC,
    _ENUM_HW_I2C_MASTER,
}STRUCT_ENUM_ACCESS_INDEIRECT_EEPROM_METHOD;

typedef enum
{
    _ENUM_INPUT = 0,
    _ENUM_PUSHPULL,
    _ENUM_OPENDRAIN,
}STRUCT_ENUM_SET_PINSHARE_TYPE;

typedef struct STRUCT_PIN_VALUE_CONTROL
{
    DWORD ulRegAddr;
    BYTE ucBitStart;
    BYTE ucBitEnd;
    STRUCT_PIN_VALUE_CONTROL() :ulRegAddr(0), ucBitStart(0), ucBitEnd(0)
    {
    }

    void Reset()
    {
        ulRegAddr = 0;
        ucBitStart = 0;
        ucBitEnd = 0;
    }
}STRUCT_PIN_VALUE_CONTROL;

typedef struct
{
    STRUCT_PIN_VALUE_CONTROL stFuncPinReg;
    BYTE ucFuncNormalConfValue;
    BYTE ucFuncI2CConfValue;
}STRUCT_GPIO_FUNC_CTRL;

typedef struct
{
    string strPinName;
    BOOL bNeedGPIOFunc;
    DWORD ulPinShareType;
    STRUCT_PIN_VALUE_CONTROL stPinshareReg;
    BYTE ucInputConf;
    BYTE ucPushpullConf;
    BYTE ucOpenDrainConf;
    STRUCT_PIN_VALUE_CONTROL stPinValCtrlReg;
    DWORD ulFuncType;
    STRUCT_GPIO_FUNC_CTRL stFuncCtrl;
}STRUCT_GPIO_PIN_CTRL;

class CCpDeviceSelSetting
{
public:
    CCpDeviceSelSetting();
    ~CCpDeviceSelSetting();

    // access EEPROM by location
    void SetDeviceLocationType(STRUCT_ENUM_DEVICE_TYPE enumType) { m_enumDeviceType = enumType; }
    STRUCT_ENUM_DEVICE_TYPE GetDeviceLocationType(void) { return m_enumDeviceType; }

    // EEPROM r/w setting
    void SetEepromSubAddrType(STRUCT_ENUM_EEPROM_ADDR_LEN enumType) { m_enumSubAddrLen = enumType; }
    STRUCT_ENUM_EEPROM_ADDR_LEN GetEepromSubAddrType(void) { return m_enumSubAddrLen; }
    void SetEepromStartAddr(DWORD ulAddr) { m_ulStartAddr = ulAddr; }
    DWORD GetEepromStartAddr(void) { return m_ulStartAddr; }
    void SetEepromSlaveAddr(BYTE ucAddr) { m_ucSlaveAddr = ucAddr; }
    BYTE GetEepromSlaveAddr(void) { return m_ucSlaveAddr; }
    void SetEepromPageLength(BYTE ucLen) { m_ucPageLen = ucLen; }
    BYTE GetEepromPageLength(void) { return m_ucPageLen; }
    void SetEepromWriteDelayTime(DWORD ulTime) { m_ulDelayTime = ulTime; }
    DWORD GetEepromWriteDelayTime(void) { return m_ulDelayTime; }

    // Direct EEPROM setting
    void SetDirectEepromNeedWP(BOOL bSetWp) { m_bNeedSetWP = bSetWp; }
    BOOL GetDirectEepromNeedWP(void) { return m_bNeedSetWP; }
    void SetDirectEepromPinshareType(STRUCT_ENUM_SET_PINSHARE_TYPE enumType) { m_enumDirectPinshare = enumType; }
    STRUCT_ENUM_SET_PINSHARE_TYPE GetDirectEepromPinshareType(void) { return m_enumDirectPinshare; }
    void SetDirectEepromWP(STRUCT_GPIO_PIN_CTRL stPin) { m_stDirectWP = stPin; }
    STRUCT_GPIO_PIN_CTRL GetDirectEepromWP(void) { return m_stDirectWP; }

    // In-direct EEPROM setting
    void SetInDirectEepromAccessMethod(STRUCT_ENUM_ACCESS_INDEIRECT_EEPROM_METHOD enumMethod) { m_enumAccessMethod = enumMethod; }
    STRUCT_ENUM_ACCESS_INDEIRECT_EEPROM_METHOD GetInDirectEepromAccessMethod(void) { return m_enumAccessMethod; }
    void SetInDirectEepromNeedChangePinshare(BOOL bCheck) { m_bNeedChangePinshareCheckSCLSDA = bCheck; }
    BOOL GetInDirectEepromNeedChangePinshare(void) { return m_bNeedChangePinshareCheckSCLSDA; }
    void SetInDirectEepromPinshareType(STRUCT_ENUM_SET_PINSHARE_TYPE enumType) { m_enumInDirectPinshare = enumType; }
    STRUCT_ENUM_SET_PINSHARE_TYPE GetInDirectEepromPinshareType(void) { return m_enumInDirectPinshare; }
    void SetInDirectEepromSCL(STRUCT_GPIO_PIN_CTRL stPin) { m_stInDirectSCL = stPin; }
    STRUCT_GPIO_PIN_CTRL GetInDirectEepromSCL(void) { return m_stInDirectSCL; }
    void SetInDirectEepromSDA(STRUCT_GPIO_PIN_CTRL stPin) { m_stInDirectSDA = stPin; }
    STRUCT_GPIO_PIN_CTRL GetInDirectEepromSDA(void) { return m_stInDirectSDA; }
    void SetInDirectEepromWP(STRUCT_GPIO_PIN_CTRL stPin) { m_stInDirectWP = stPin; }
    STRUCT_GPIO_PIN_CTRL GetInDirectEepromWP(void) { return m_stInDirectWP; }
    void SetInDirectEepromTryCount(BYTE ucCount) { m_ucTryCount = ucCount; }
    BYTE GetInDirectEepromTryCount(void) { return m_ucTryCount; }
    void SetInDirectEepromIICScl(STRUCT_GPIO_PIN_CTRL stPin) { m_stIICScl = stPin; }
    STRUCT_GPIO_PIN_CTRL GetInDirectEepromIICScl(void) { return m_stIICScl; }
    void SetInDirectEepromIICSda(STRUCT_GPIO_PIN_CTRL stPin) { m_stIICSda = stPin; }
    STRUCT_GPIO_PIN_CTRL GetInDirectEepromIICSda(void) { return m_stIICSda; }

    // Flash setting
    void SetFlashBankIndex(BYTE ucBank) { m_ucBankIndex = ucBank; }
    BYTE GetFlashBankIndex(void) { return m_ucBankIndex; }
    void SetFlashStartAddr(DWORD uAddr) { m_ulFlashAddr = uAddr; }
    DWORD GetFlashStartAddr(void) { return m_ulFlashAddr; }

private:
    // access EEPROM by location
    STRUCT_ENUM_DEVICE_TYPE m_enumDeviceType;

    // EEPROM r/w setting
    STRUCT_ENUM_EEPROM_ADDR_LEN m_enumSubAddrLen;
    DWORD m_ulStartAddr;
    BYTE m_ucSlaveAddr;
    BYTE m_ucPageLen;
    DWORD m_ulDelayTime;

    // Direct EEPROM setting
    BOOL m_bNeedSetWP;
    STRUCT_ENUM_SET_PINSHARE_TYPE m_enumDirectPinshare;
    STRUCT_GPIO_PIN_CTRL m_stDirectWP;

    // In-direct EEPROM setting
    STRUCT_ENUM_ACCESS_INDEIRECT_EEPROM_METHOD m_enumAccessMethod;
    BOOL m_bNeedChangePinshareCheckSCLSDA;
    STRUCT_ENUM_SET_PINSHARE_TYPE m_enumInDirectPinshare;
    STRUCT_GPIO_PIN_CTRL m_stInDirectSCL;
    STRUCT_GPIO_PIN_CTRL m_stInDirectSDA;
    STRUCT_GPIO_PIN_CTRL m_stInDirectWP;
    BYTE m_ucTryCount;
    STRUCT_GPIO_PIN_CTRL m_stIICScl;
    STRUCT_GPIO_PIN_CTRL m_stIICSda;

    // Flash setting
    BYTE m_ucBankIndex;
    DWORD m_ulFlashAddr;
};
//////////////////////////////////////////////////////////////////////////

class CCpIspFlowSetting
{
public:
    CCpIspFlowSetting();
    ~CCpIspFlowSetting();

    void SetCommID(STRUCT_ENUM_COMM_ID enumComm) { m_enumCommID = enumComm; }
    STRUCT_ENUM_COMM_ID GetCommId(void) { return m_enumCommID; }

    void SetReadVerSetting(STRUCT_READ_VER_SETTING stVerSetting) { m_stReadVer = stVerSetting; }
    STRUCT_READ_VER_SETTING GetReadVerSetting(void) { return m_stReadVer; }

    DWORD GetProgramFwCount(void) { return (DWORD)m_mapFw.size(); }
    void GetProgramFwMap(std::map<DWORD, STRUCT_FW_BANK_SETTING, sortAscending<DWORD> > &mapFw) { mapFw = m_mapFw; }
    void SetProgramFwMap(std::map<DWORD, STRUCT_FW_BANK_SETTING, sortAscending<DWORD> > &mapFw) { m_mapFw = mapFw; }

    void SetIspFunctionType(const ENUM_ISP_FUNCTION_TYPE enumType) { m_enumFuncType = enumType; }
    ENUM_ISP_FUNCTION_TYPE GetIspFunctionType(void) { return m_enumFuncType; }

    void SetProgramStartBank(const DWORD ulBank) { m_ulProgStartBank = ulBank; }
    DWORD GetProgramStartBank(void) { return m_ulProgStartBank; }

    // Basic setting
    void SetAutoDetectSlave(const BOOL bFlag) { m_bAutoDetectSlave = bFlag; }
    BOOL GetAutoDetectSlave(void) { return m_bAutoDetectSlave; }

    void SetSpecifiedSlave(const BYTE ucSlave) { m_ucSpecifiedSlave = ucSlave; }
    BYTE GetSpecifiedSlave(void) { return m_ucSpecifiedSlave; }

    void SetIspBeforeAcOn(const BOOL bFlag) { m_bIspBeforeAcOn = bFlag; }
    BOOL GetIspBeforeAcOn(void) { return m_bIspBeforeAcOn; }

    void SetTimeBeforeAcOn(const DWORD ulTime) { m_ulTimeBeforeAcOn = ulTime; }
    DWORD GetTimeBeforeAcOn(void) { return m_ulTimeBeforeAcOn; }

    void SetResetMcu(const BOOL bFlag) { m_bResetMcu = bFlag; }
    BOOL GetResetMcu(void) { return m_bResetMcu; }

    void SetIspSpeed(const DWORD ulSpeed) { m_ulIspSpeed = ulSpeed; }
    DWORD GetIspSpeed(void) { return m_ulIspSpeed; }

    void SetSaveResult(const BOOL bFlag) { m_bSaveResult = bFlag; }
    BOOL GetSaveResult(void) { return m_bSaveResult; }

    void SetSaveDebugLog(const BOOL bFlag) { m_bSaveDebugLog = bFlag; }
    BOOL GetSaveDebugLog(void) { return m_bSaveDebugLog; }

    void SetSaveProgress(const BOOL bFlag) { m_bSaveProgress = bFlag; }
    BOOL GetSaveProgress(void) { return m_bSaveProgress; }

    void SetTimeAfterReset(const DWORD ulTime) { m_ulTimeAfterReset = ulTime; }
    DWORD GetTimeAfterReset(void) { return m_ulTimeAfterReset; }

    // Flash type setting
    void SetAutoDetectFlash(const BOOL bFlag) { m_bAutoDetectFlash = bFlag; }
    BOOL GetAutoDetectFlash(void) { return m_bAutoDetectFlash; }

    void SetSpecifiedFlashName(const std::string strName) { m_strSpecifiedFlashName = strName; }
    std::string GetSpecifiedFlashName(void) { return m_strSpecifiedFlashName; }

    void SetFlashMismatch(const ENUM_SPECIFY_FLASH_MISMATCH_DETECTED enumOperate) { m_enumFlashMismatch = enumOperate; }
    ENUM_SPECIFY_FLASH_MISMATCH_DETECTED GetFlashMismatch(void) { return m_enumFlashMismatch; }

    // Erase setting
    void SetEraseType(const ENUM_ERASE_FLASH_TYPE enumOperate) { m_enumEraseType = enumOperate; }
    ENUM_ERASE_FLASH_TYPE GetEraseType(void) { return m_enumEraseType; }

    // Reserved Setting
    DWORD GetReservedItemCount(void) { return (DWORD)m_mapReservedItem.size(); }
    void GetReservedItemMap(std::map<DWORD, STRUCT_RESERVE_ITEM> &mapReservedItem) { mapReservedItem = m_mapReservedItem; }
    void SetReservedItemMap(std::map<DWORD, STRUCT_RESERVE_ITEM> &mapReservedItem) { m_mapReservedItem = mapReservedItem; }

    // Boot flag
    void SetBootFlag(const STRUCT_BOOT_FLAG_SETTING &stFlag) { m_stBootFlag = stFlag; }
    STRUCT_BOOT_FLAG_SETTING GetBootFlag(void) { return m_stBootFlag; }

    // Customized setting
    void SetQisdaFactory(const BOOL bFlag) { m_bQisdaFactory = bFlag; }
    BOOL GetQisdaFactory(void) { return m_bQisdaFactory; }

    void Reset();
private:
    STRUCT_ENUM_COMM_ID m_enumCommID;
    STRUCT_READ_VER_SETTING m_stReadVer;

    // Fw setting
    std::map<DWORD, STRUCT_FW_BANK_SETTING, sortAscending<DWORD> > m_mapFw;
    ENUM_ISP_FUNCTION_TYPE m_enumFuncType;
    DWORD m_ulProgStartBank;

    // Basic setting
    BOOL m_bAutoDetectSlave;
    BYTE m_ucSpecifiedSlave;
    BOOL m_bIspBeforeAcOn;
    DWORD m_ulTimeBeforeAcOn;
    BOOL m_bResetMcu;
    DWORD m_ulIspSpeed;
    BOOL m_bSaveResult;
    BOOL m_bSaveDebugLog;
    BOOL m_bSaveProgress;
    DWORD m_ulTimeAfterReset;

    // Flash type setting
    BOOL m_bAutoDetectFlash;
    std::string m_strSpecifiedFlashName;
    ENUM_SPECIFY_FLASH_MISMATCH_DETECTED m_enumFlashMismatch;

    // Erase setting
    ENUM_ERASE_FLASH_TYPE m_enumEraseType;

    // Reserved Setting
    std::map<DWORD, STRUCT_RESERVE_ITEM> m_mapReservedItem;

    // Boot flag
    STRUCT_BOOT_FLAG_SETTING m_stBootFlag;

    // Customized setting
    BOOL m_bQisdaFactory;
};

class CCpDbIspFlowSetting : public CCpDbBase
{
public:
    CCpDbIspFlowSetting();
    ~CCpDbIspFlowSetting();

    BOOL SaveDB(string strDBFile, CCpIspFlowSetting& cpSetting);
    BOOL LoadDB(string strDBFile, CCpIspFlowSetting& cpSetting);


private:
    BOOL LoadDBInfo(LPBYTE pucBuf, CCpIspFlowSetting& cpSetting);
    BOOL LoadProgramFwDB(LPBYTE pucBuf, DWORD& ulOffset, CCpIspFlowSetting& cpSetting);
    BOOL LoadReservedItemDB(LPBYTE pucBuf, DWORD& ulOffset, CCpIspFlowSetting& cpSetting);
    BOOL SaveProgramFwDB(std::map<DWORD, STRUCT_FW_BANK_SETTING, sortAscending<DWORD> > &mapFw, BYTE*& pucBuff, DWORD& ulBuffIndex, DWORD& ulFileLength);
    BOOL SaveReservedItemDB(std::map<DWORD, STRUCT_RESERVE_ITEM> &mapReservedItem, BYTE*& pucBuff, DWORD& ulBuffIndex, DWORD& ulFileLength);
};


#define BYTES_TO_DWORD(ucsz) (ucsz[3] + (ucsz[2] << 8) + (ucsz[1] << 16) + (ucsz[0] << 24))
#define BYTES_TO_WORD(ucsz) (ucsz[1] + (ucsz[0] << 8))
#define DWORD_TO_BYTES(ulValue, ucsz) {ucsz[3] = ulValue & 0xFF; ucsz[2] = (ulValue >> 8) & 0xFF; ucsz[1] = (ulValue >> 16) & 0xFF; ucsz[0] = (ulValue >> 24) & 0xFF;}
#define WORD_TO_BYTES(usValue, ucsz) {ucsz[1] = usValue & 0xFF; ucsz[0] = (usValue >> 8) & 0xFF;}

#endif
