#include "stdafx.h"
#include "CpIspShare.h"
#include "../../Tool_Common/CpRsFuncBase.h"

#define US_ENTER_ISP_RETRY_TIME 10

extern CCpIspFlowSetting *g_pcpIspSetting;
extern CCpRsString g_cpstrPath;
extern CCpIspProgress g_cpIspProgress;

STRUCT_ENUM_ERROR_TYPE DoIsp()
{
	CCpRsString strLog;          
	BYTE *pucFWBuff = NULL;              // Bin Buffer            
	BOOL bSectorErase = 0; 
	DWORD ulSectorEraseCount = 0;
	DWORD ulProLen = 0;                 // Bank Valid Length  
	DWORD i = 0;
	BYTE ucszReadGD25M21BSecurityRegs[N_GD25M21B_SECURITY_REG_COUNT] = { 0 };
	STRUCT_ENUM_ERROR_TYPE enumError = _ERROR_SUCCESS;
	ENUM_ISP_FUNCTION_TYPE enumFuncType = _ENUM_FUNC_UNKNOWN;
	CCpFlash cpFlash;
	std::map<DWORD, STRUCT_FW_BANK_SETTING, sortAscending<DWORD> > mapFw;
	std::map<DWORD, STRUCT_FW_BANK_SETTING>::reverse_iterator Reverse_itMapFw;
	std::map<DWORD, STRUCT_RESERVE_ITEM> mapReservedItem;
	std::map<string, CCpFlash> mapFlashInfo;
	std::multimap<string, string> mapIDCodeFlashName;
	string strResultPath;
    STRUCT_ROM_FLAG_SETTING stRomFlag;

	g_pcpIspSetting->GetProgramFwMap(mapFw);
	g_pcpIspSetting->GetReservedItemMap(mapReservedItem);
	enumFuncType = g_pcpIspSetting->GetIspFunctionType();


	strLog.Format(_T("......Do ISP Begin......"));
	LogRecord(strLog);

	// Get Flash Map Info
	LoadFlashData(mapFlashInfo, g_cpstrPath);
	if(mapFlashInfo.empty())
	{
		return _ERROR_PLUGIN_ISP_FLASH_MAP_EMPTY;
	}

	// Check MapFw (If Function Type Need ISP)
	if((mapFw.empty()) && (enumFuncType != _ENUM_FUNC_ERASE))
	{
		return _ERROR_PLUGIN_ISP_FW_MAP_EMPTY;
	}

	// Set I2C Speed 200 Befor Enter ISP Mode
	enumError = SetI2CSpeed(200);

	// Get Slave Addr, EnterIspMode
	enumError = EnterIspMode(US_ENTER_ISP_RETRY_TIME);
	IF_ERROR_THEN_RETURN(enumError);

	// Set MCU Reg Default Value
	enumError = SetMcuRegDefaultValue();
	IF_ERROR_THEN_RETURN(enumError);

	// Set I2C Speed
	enumError = SetI2CSpeed((WORD)g_pcpIspSetting->GetIspSpeed());

	// Get Flash Type
	enumError = GetFlashType(mapFlashInfo, cpFlash);
	if(_ERROR_PLUGIN_ISP_FLASH_MISMATCH_USE_DEFAULT_VALUE == enumError)
	{
		strLog.Format(_T("Flash MisMatch Use Default Value"));
		LogRecord(strLog);
	}
	
	strLog.Format(_T("Flash Name: %s"), cpFlash.GetFlashName().c_str());
	ShowMsg(strLog);

	// Judge whether the memory space of bank is enough
	if(_ENUM_FUNC_ERASE != enumFuncType || _ENUM_FLASH_CHIP_ERASE != g_pcpIspSetting->GetEraseType())
	{
		Reverse_itMapFw = mapFw.rbegin();
		if((Reverse_itMapFw->first) > (DWORD)(cpFlash.GetBankCount() - 1))
		{
			return _ERROR_PLUGIN_ISP_BANK_NO_ENOUGH;
		}
	}

	enumError = SetWPPin();
	if(_ERROR_SUCCESS != enumError)
	{
		strLog.Format(_T(" Set WP Pin Fail! "));
		LogRecord(strLog);
	}

	// Disable WP
	enumError = DisableSoftWareWP(cpFlash, ucszReadGD25M21BSecurityRegs);
	if(_ERROR_SUCCESS != enumError)
	{
		strLog.Format(_T(" Disable WP Fail! "));
		LogRecord(strLog);
	}

	// Chip Erase Or Bank Erase
	enumError = EraseOperation(cpFlash, ucszReadGD25M21BSecurityRegs, mapReservedItem);
	if(_ERROR_SUCCESS != enumError)
	{
		strLog.Format(_T(" EraseOperation Fail! "));
		LogRecord(strLog);
	}

	if(_ENUM_FUNC_ERASE == enumFuncType)
	{
		return enumError;
	}
    
	if(IS_BOOT_FUNC_TYPE(enumFuncType))
	{
    	stRomFlag.ulAddr = 0x7000;
    	memset(stRomFlag.ucszFlag, 0xff, UL_BOOT_FLAG_COUNT);
    	stRomFlag.ucszFlag[0] = 0x55;
    	stRomFlag.ucszFlag[1] = 0xAA;
    	stRomFlag.ucRomBank = 1;
		enumError = CheckRomFlagOperation(TRUE, cpFlash, stRomFlag);
		if(_ERROR_SUCCESS != enumError)
		{
			strLog.Format(_T("Clear Rom Flag Fail! "));
			LogRecord(strLog);
			return enumError;
		}
	}
	else if(IS_USER_FUNC_TYPE(enumFuncType))
	{
		// Clear User Flag
		enumError = UserFlagOperation(TRUE, cpFlash);
		if(_ERROR_SUCCESS != enumError)
		{
			strLog.Format(_T(" Clear User Flag Fail! "));
			LogRecord(strLog);
			return enumError;
		}

		if(IS_CLEAR_EXTRA_FLAG_FUNC_TYPE(enumFuncType))
		{
			// CopyBank or CopyBankSW Write 0x7004 [0x55]
			enumError = WriteExtraBootFlag(cpFlash);
			if(_ERROR_SUCCESS != enumError)
			{
				strLog.Format(_T(" WriteExtraBootFlag Fail! "));
				LogRecord(strLog);
				return enumError;
			}
		}
	}
	// ISP
	if(!g_cpIspProgress.SetRange(0, (g_pcpIspSetting->GetProgramFwCount() * UL_PAGE_COUNT_PER_BANK)))
	{
		strLog.Format(_T(" Isp Progress Set Range Fail! "));
		LogRecord(strLog);
		return _ERROR_PLUGIN_ISP_PROGRESS_SET_RANGE;
	}
	g_cpIspProgress.SetStep(1);
	g_cpIspProgress.SetPrintSwitch(TRUE);
	g_cpIspProgress.SetRecordResultSwitch(g_pcpIspSetting->GetSaveProgress());


	pucFWBuff = new BYTE[UL_MAX_SERIALBANK_LENGTH];
	for(Reverse_itMapFw = mapFw.rbegin(); Reverse_itMapFw != mapFw.rend(); ++Reverse_itMapFw)
	{
		memset(pucFWBuff, 0xff, UL_MAX_SERIALBANK_LENGTH);

		ulProLen = UL_MAX_SERIALBANK_LENGTH;

		enumError = GetFinalCode(Reverse_itMapFw->first, Reverse_itMapFw->second, pucFWBuff, &ulProLen, &bSectorErase, &ulSectorEraseCount, mapReservedItem);
		if(_ERROR_SUCCESS != enumError)
		{
			SAFELY_DELETE(pucFWBuff);
			strLog.Format(_T(" Bank %d Get Final Code Fail! "), Reverse_itMapFw->first);
			LogRecord(strLog);
			return enumError;
		}

		// Bank Erase Or Sector Erase
		if(_ENUM_FLASH_CHIP_ERASE != g_pcpIspSetting->GetEraseType())
		{
			if(bSectorErase)
			{
				for(i = 0;  i < ulSectorEraseCount; i++)
				{
					enumError = SectorErase(Reverse_itMapFw->first, i, &cpFlash);
					if(_ERROR_SUCCESS != enumError)
					{
						SAFELY_DELETE(pucFWBuff);
						strLog.Format(_T(" Sector %d Erase In Bank %d Fail! "), i, Reverse_itMapFw->first);
						LogRecord(strLog);
						return enumError;
					}
				}
			}
			else
			{
				enumError = BankErase(Reverse_itMapFw->first, &cpFlash);
				if(_ERROR_SUCCESS != enumError)
				{
					SAFELY_DELETE(pucFWBuff);
					strLog.Format(_T(" Bank %d Bank Erase Fail! "), Reverse_itMapFw->first);
					LogRecord(strLog);
					return enumError;
				}
			}
		}

		strLog.Format(_T("Bank %d Erase Success"), Reverse_itMapFw->first);
		LogRecord(strLog);

		enumError = WriteDataToFlash((Reverse_itMapFw->first) << 16, ulProLen, pucFWBuff, &cpFlash);
		if(_ERROR_SUCCESS != enumError)
		{
			SAFELY_DELETE(pucFWBuff);
			strLog.Format(_T(" Bank %d Write Data To Flash Fail! "), Reverse_itMapFw->first);
			LogRecord(strLog);
			return enumError;
		}
		strLog.Format(_T("Bank %d ISP Success"), Reverse_itMapFw->first);
		LogRecord(strLog);
	}
	if(g_cpIspProgress.GetPrintSwitch())
	{
		//Gotoxy(0, 1000);
		cout << "\rISP Progress: 100%" << endl;
		fflush(stdout);
	}

	strResultPath.assign(g_cpstrPath.begin(), g_cpstrPath.end());
	g_cpIspProgress.ReocordISPResult(strResultPath, 100);

	g_cpIspProgress.SetPrintSwitch(FALSE);
	g_cpIspProgress.SetRecordResultSwitch(FALSE);

	if(IS_BOOT_FUNC_TYPE(enumFuncType))
	{
	    enumError = CheckRomFlagOperation(FALSE, cpFlash, stRomFlag);
		if(_ERROR_SUCCESS != enumError)
		{
			SAFELY_DELETE(pucFWBuff);
			strLog.Format(_T(" Write Rom Flag Fail! "));
			LogRecord(strLog);
			return enumError;
		}
	}
	else if(IS_USER_FUNC_TYPE(enumFuncType))
	{
		// Write User Flag
		enumError = UserFlagOperation(FALSE, cpFlash);
		if(_ERROR_SUCCESS != enumError)
		{
			strLog.Format(_T(" Write User Flag Fail! "));
			LogRecord(strLog);
			return enumError;
		}
	}

	SAFELY_DELETE(pucFWBuff);

	// Enable WP
	enumError = EnableSoftWareWP(cpFlash, ucszReadGD25M21BSecurityRegs);
	if(_ERROR_SUCCESS != enumError)
	{
		strLog.Format(_T(" EnableSoftWareWP Fail!, Error Code:0x%x"), enumError);
		LogRecord(strLog);
	}

	// Reset MCU
	ResetMcuAsSetting();

	strLog.Format(_T("......Do ISP End......"));
	LogRecord(strLog);

	return _ERROR_SUCCESS;
}

STRUCT_ENUM_ERROR_TYPE ReadFWVersion()
{
	STRUCT_READ_FW_VERSION stReadFWVer = { 0 };
	STRUCT_ENUM_ERROR_TYPE enumError = _ERROR_SUCCESS;
	CHAR cszTemp[10] = { 0 };
	CHAR cszCurrentDir[MAX_DIR_LENGTH] = { 0 };
	string strPathPlusName;
	CHAR cszIspVersion[] = "ISPVersion.txt";
	CHAR cszReadVersion[] = "ReadVersion";
	CHAR cszProjectID[] = "ProjectID";
	CHAR cszFWVersion[] = "FWVersion";
	CHAR cszFWSubVersion[] = "FWSubVersion";
	strPathPlusName.assign(g_cpstrPath.begin(), g_cpstrPath.end());
	strPathPlusName += cszIspVersion;
	strncpy(cszCurrentDir, strPathPlusName.c_str(), MAX_DIR_LENGTH);

	enumError = GetFWVersion(&stReadFWVer);
	if(_ERROR_SUCCESS != enumError)
	{
	    printf("Get FW Version Fail, Error Code: 0x%x\n", enumError);
		return enumError;
	}

	if(!IS_COPY_BANK_FUNC_TYPE(g_pcpIspSetting->GetIspFunctionType()))
	{
		sprintf(cszTemp, "0x%x", stReadFWVer.usProjectID);
		if(g_pcpIspSetting->GetSaveResult())
		{
			CpRsWritePrivateProfileString(cszReadVersion, cszProjectID, cszTemp, cszCurrentDir);
		}
		cout << "ProjectID: " << cszTemp << endl;
	}


	memset(cszTemp, 0, 10);
	sprintf(cszTemp, "0x%02x", stReadFWVer.ucFWVer);
	if(g_pcpIspSetting->GetSaveResult())
	{	
		CpRsWritePrivateProfileString(cszReadVersion, cszFWVersion, cszTemp, cszCurrentDir);
	}
	cout << "FWVersion: " << cszTemp << endl;

	memset(cszTemp, 0, 10);
	sprintf(cszTemp, "0x%02x", stReadFWVer.ucFWSubVer);
	if(g_pcpIspSetting->GetSaveResult())
	{	
		CpRsWritePrivateProfileString(cszReadVersion, cszFWSubVersion, cszTemp, cszCurrentDir);
	}
	cout << "FWSubVersion: " << cszTemp << endl;

	return enumError;
}

